@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 """
-Command line interface for the Mergin client module. When installed with pip, this script
+Command line interface for the Mergin Maps client module. When installed with pip, this script
 is installed as 'mergin' command line tool (defined in setup.py). If you have installed the module
 but the tool is not available, you may need to fix your PATH (e.g. add ~/.local/bin where
 pip puts these tools).
@@ -31,10 +31,8 @@ from mergin.client_pull import (
     download_project_finalize,
     download_project_is_running,
 )
-from mergin.client_pull import pull_project_async, pull_project_is_running, pull_project_finalize, \
-    pull_project_cancel
-from mergin.client_push import push_project_async, push_project_is_running, push_project_finalize, \
-    push_project_cancel
+from mergin.client_pull import pull_project_async, pull_project_is_running, pull_project_finalize, pull_project_cancel
+from mergin.client_push import push_project_async, push_project_is_running, push_project_finalize, push_project_cancel
 
 
 from pygeodiff import GeoDiff
@@ -108,7 +106,7 @@ def get_token(url, username, password):
 
 
 def get_client(url=None, auth_token=None, username=None, password=None):
-    """Return Mergin client."""
+    """Return Mergin Maps client."""
     if auth_token is not None:
         try:
             mc = MerginClient(url, auth_token=auth_token)
@@ -143,20 +141,22 @@ def _print_unhandled_exception():
         click.echo(line)
 
 
-@click.group(epilog=f"Copyright (C) 2019-2021 Lutra Consulting\n\n(mergin-py-client v{__version__} / pygeodiff v{GeoDiff().version()})")
+@click.group(
+    epilog=f"Copyright (C) 2019-2021 Lutra Consulting\n\n(mergin-py-client v{__version__} / pygeodiff v{GeoDiff().version()})"
+)
 @click.option(
     "--url",
     envvar="MERGIN_URL",
     default=MerginClient.default_url(),
-    help=f"Mergin server URL. Default is: {MerginClient.default_url()}",
+    help=f"Mergin Maps server URL. Default is: {MerginClient.default_url()}",
 )
-@click.option("--auth-token", envvar="MERGIN_AUTH", help="Mergin authentication token string")
+@click.option("--auth-token", envvar="MERGIN_AUTH", help="Mergin Maps authentication token string")
 @click.option("--username", envvar="MERGIN_USERNAME")
 @click.option("--password", cls=OptionPasswordIfUser, prompt=True, hide_input=True, envvar="MERGIN_PASSWORD")
 @click.pass_context
 def cli(ctx, url, auth_token, username, password):
     """
-    Command line interface for the Mergin client module.
+    Command line interface for the Mergin Maps client module.
     For user authentication on server there are two options:
 
      1. authorization token environment variable (MERGIN_AUTH) is defined, or
@@ -183,12 +183,15 @@ def login(ctx):
 @cli.command()
 @click.argument("project")
 @click.option("--public", is_flag=True, default=False, help="Public project, visible to everyone")
-@click.option("--from-dir", default=None,
-              help="Content of the directory will be uploaded to the newly created project. "
-                   "The directory will get assigned to the project.")
+@click.option(
+    "--from-dir",
+    default=None,
+    help="Content of the directory will be uploaded to the newly created project. "
+    "The directory will get assigned to the project.",
+)
 @click.pass_context
 def create(ctx, project, public, from_dir):
-    """Create a new project on Mergin server. `project` needs to be a combination of namespace/project."""
+    """Create a new project on Mergin Maps server. `project` needs to be a combination of namespace/project."""
     mc = ctx.obj["client"]
     if mc is None:
         return
@@ -223,16 +226,36 @@ def create(ctx, project, public, from_dir):
     help="What kind of projects (e.g. 'created' for just my projects,"
     "'shared' for projects shared with me. No flag means returns all public projects.",
 )
+@click.option(
+    "--name",
+    help="Filter projects with name like name",
+)
+@click.option(
+    "--namespace",
+    help="Filter projects with namespace like namespace",
+)
+@click.option(
+    "--order_params",
+    help="optional attributes for sorting the list. "
+    "It should be a comma separated attribute names "
+    "with _asc or _desc appended for sorting direction. "
+    'For example: "namespace_asc,disk_usage_desc". '
+    "Available attrs: namespace, name, created, updated, disk_usage, creator",
+)
 @click.pass_context
-def list_projects(ctx, flag):
+def list_projects(ctx, flag, name, namespace, order_params):
     """List projects on the server."""
     filter_str = "(filter flag={})".format(flag) if flag is not None else "(all public)"
+
     click.echo("List of projects {}:".format(filter_str))
+
     mc = ctx.obj["client"]
     if mc is None:
         return
-    resp = mc.paginated_projects_list(flag=flag)
-    projects_list = resp["projects"]
+
+    projects_list = mc.projects_list(flag=flag, name=name, namespace=namespace, order_params=order_params)
+
+    click.echo("Fetched {} projects .".format(len(projects_list)))
     for project in projects_list:
         full_name = "{} / {}".format(project["namespace"], project["name"])
         click.echo(
@@ -332,10 +355,8 @@ def download_file(ctx, filepath, output, version):
     mc = ctx.obj["client"]
     if mc is None:
         return
-    mp = MerginProject(os.getcwd())
-    project_path = mp.metadata["name"]
     try:
-        job = download_file_async(mc, project_path, filepath, output, version)
+        job = download_file_async(mc, os.getcwd(), filepath, output, version)
         with click.progressbar(length=job.total_size) as bar:
             last_transferred_size = 0
             while download_project_is_running(job):
@@ -377,10 +398,13 @@ def status(ctx):
         _print_unhandled_exception()
         return
 
-    if mc.has_unfinished_pull():
-        click.secho("The previous pull has not finished completely: status "
-                    "of some files may be reported incorrectly. Use "
-                    "resolve_unfinished_pull command to try to fix that.", fg="yellow")
+    if mc.has_unfinished_pull(os.getcwd()):
+        click.secho(
+            "The previous pull has not finished completely: status "
+            "of some files may be reported incorrectly. Use "
+            "resolve_unfinished_pull command to try to fix that.",
+            fg="yellow",
+        )
 
     click.secho("### Server changes:", fg="magenta")
     pretty_diff(pull_changes)
@@ -393,7 +417,7 @@ def status(ctx):
 @cli.command()
 @click.pass_context
 def push(ctx):
-    """Upload local changes into Mergin repository."""
+    """Upload local changes into Mergin Maps repository."""
     mc = ctx.obj["client"]
     if mc is None:
         return
@@ -425,7 +449,7 @@ def push(ctx):
 @cli.command()
 @click.pass_context
 def pull(ctx):
-    """Fetch changes from Mergin repository."""
+    """Fetch changes from Mergin Maps repository."""
     mc = ctx.obj["client"]
     if mc is None:
         return
@@ -503,7 +527,7 @@ def show_file_history(ctx, path):
 @click.argument("version")
 @click.pass_context
 def show_file_changeset(ctx, path, version):
-    """ Displays information about project changes."""
+    """Displays information about project changes."""
     mc = ctx.obj["client"]
     if mc is None:
         return
