@@ -12,6 +12,7 @@ import platform
 from datetime import datetime, timezone
 import dateutil.parser
 import ssl
+from enum import Enum, auto
 import re
 
 from .common import ClientError, LoginError, InvalidProject
@@ -37,6 +38,13 @@ class TokenError(Exception):
     pass
 
 
+class ServerType(Enum):
+    OLD = auto()  # Server is old and does not support workspaces
+    CE = auto()  # Server is Community Edition
+    EE = auto()  # Server is Enterprise Edition
+    SAAS = auto()  # Server is SaaS
+
+
 def decode_token_data(token):
     token_prefix = "Bearer ."
     if not token.startswith(token_prefix):
@@ -70,6 +78,7 @@ class MerginClient:
         self._auth_params = None
         self._auth_session = None
         self._user_info = None
+        self._server_type = None
         self.client_version = "Python-client/" + __version__
         if plugin_version is not None:  # this could be e.g. "Plugin/2020.1 QGIS/3.14"
             self.client_version += " " + plugin_version
@@ -310,6 +319,7 @@ class MerginClient:
         Requests information about user from /user/service endpoint if such exists in self.url server.
 
         Returns response from server as JSON dict or None if endpoint is not found
+        This can be removed once our SaaS server is upgraded to support workspaces
         """
 
         try:
@@ -322,6 +332,76 @@ class MerginClient:
 
         return response
 
+    def workspace_service(self, workspace_id):
+        """
+        This Requests information about a workspace service from /workspace/{id}/service endpoint,
+        if such exists in self.url server.
+
+        Returns response from server as JSON dict or None if endpoint is not found
+        """
+
+        try:
+            response = self.get(f"/v1/workspace/{workspace_id}/service")
+        except ClientError as e:
+            self.log.debug(f"Unable to query for /workspace/{workspace_id}/service endpoint")
+            return
+
+        response = json.loads(response.read())
+
+        return response
+
+    def server_type(self):
+        """
+        Returns the deployment type of the server
+
+        The value is cached for self's lifetime
+
+        :returns: ServerType of server deployment
+        :rtype: ServerType
+        """
+        if not self._server_type:
+            try:
+                resp = self.get("/config")
+                config = json.load(resp)
+                if config["server_type"] == "ce":
+                    self._server_type = ServerType.CE
+                elif config["server_type"] == "ee":
+                    self._server_type = ServerType.EE
+                elif config["server_type"] == "saas":
+                    self._server_type = ServerType.SAAS
+            except (ClientError, KeyError):
+                self._server_type = ServerType.OLD
+
+        return self._server_type
+
+    def workspaces_list(self):
+        """
+        Find all available workspaces
+
+        :rtype: List[Dict]
+        """
+        resp = self.get("/v1/workspaces")
+        workspaces = json.load(resp)
+        return workspaces
+
+    def create_workspace(self, workspace_name):
+        """
+        Create new workspace for currently active user.
+
+        :param workspace_name: Workspace name to create
+        :type workspace_name: String
+        """
+        if not self._user_info:
+            raise Exception("Authentication required")
+
+        params = {"name": workspace_name}
+
+        try:
+            self.post("/v1/workspace", params, {"Content-Type": "application/json"})
+        except Exception as e:
+            detail = f"Workspace name: {workspace_name}"
+            raise ClientError(str(e), detail)
+
     def create_project(self, project_name, is_public=False, namespace=None):
         """
         Create new project repository in user namespace on Mergin Maps server.
@@ -359,14 +439,25 @@ class MerginClient:
             namespace = self.username()
         self.create_project(project_name, is_public, namespace)
         if directory:
-            mp = MerginProject(directory)
             full_project_name = "{}/{}".format(namespace, project_name)
-            mp.metadata = {"name": full_project_name, "version": "v0", "files": []}
+            project_info = self.project_info(full_project_name)
+            mp = MerginProject(directory)
+            mp.metadata = {"name": full_project_name, "version": "v0", "files": [], "project_id": project_info["id"]}
             if mp.inspect_files():
                 self.push_project(directory)
 
     def paginated_projects_list(
-        self, page=1, per_page=50, tags=None, user=None, flag=None, name=None, namespace=None, order_params=None
+        self,
+        page=1,
+        per_page=50,
+        tags=None,
+        user=None,
+        flag=None,
+        name=None,
+        only_namespace=None,
+        namespace=None,
+        order_params=None,
+        only_public=None,
     ):
         """
         Find all available Mergin Maps projects.
@@ -383,6 +474,9 @@ class MerginClient:
         :param name: Filter projects with name like name
         :type name: String
 
+        :param only_namespace: Filter projects with namespace exactly equal to namespace
+        :type namespace: String
+
         :param namespace: Filter projects with namespace like namespace
         :type namespace: String
 
@@ -397,6 +491,9 @@ class MerginClient:
             Available attrs: namespace, name, created, updated, disk_usage, creator
         :type order_params: String
 
+        :param only_public: Only fetch public projects
+        :type only_public: Bool
+
         :rtype: List[Dict]
         """
         params = {}
@@ -408,8 +505,12 @@ class MerginClient:
             params["flag"] = flag
         if name:
             params["name"] = name
-        if namespace:
+        if only_namespace:
+            params["only_namespace"] = only_namespace
+        elif namespace:
             params["namespace"] = namespace
+        if only_public:
+            params["only_public"] = only_public
         params["page"] = page
         params["per_page"] = per_page
         if order_params is not None:
@@ -418,7 +519,17 @@ class MerginClient:
         projects = json.load(resp)
         return projects
 
-    def projects_list(self, tags=None, user=None, flag=None, name=None, namespace=None, order_params=None):
+    def projects_list(
+        self,
+        tags=None,
+        user=None,
+        flag=None,
+        name=None,
+        only_namespace=None,
+        namespace=None,
+        order_params=None,
+        only_public=None,
+    ):
         """
         Find all available Mergin Maps projects.
 
@@ -436,6 +547,9 @@ class MerginClient:
         :param name: Filter projects with name like name
         :type name: String
 
+        :param only_namespace: Filter projects with namespace exactly equal to namespace
+        :type namespace: String
+
         :param namespace: Filter projects with namespace like namespace
         :type namespace: String
 
@@ -444,6 +558,9 @@ class MerginClient:
             Available attrs: namespace, name, created, updated, disk_usage, creator
         :type order_params: String
 
+        :param only_public: Only fetch public projects
+        :type only_public: Bool
+
         :rtype: List[Dict]
         """
         projects = []
@@ -457,8 +574,10 @@ class MerginClient:
                 user=user,
                 flag=flag,
                 name=name,
+                only_namespace=only_namespace,
                 namespace=namespace,
                 order_params=order_params,
+                only_public=only_public,
             )
             fetched_projects += len(resp["projects"])
             count = resp["count"]
@@ -545,19 +664,12 @@ class MerginClient:
         download_project_wait(job)
         download_project_finalize(job)
 
-    def enough_storage_available(self, data):
-        info = self.user_info()
-        free_space = int(info["storage_limit"]) - int(info["disk_usage"])
-        upload_files_sizes = [f["size"] for f in data["updated"] + data["added"]]
-        size_to_upload = sum(upload_files_sizes)
-
-        if size_to_upload > free_space:
-            return False, free_space
-
-        return True, free_space
-
     def user_info(self):
-        resp = self.get("/v1/user/" + self.username())
+        server_type = self.server_type()
+        if server_type == ServerType.OLD:
+            resp = self.get("/v1/user/" + self.username())
+        else:
+            resp = self.get("/v1/user/profile")
         return json.load(resp)
 
     def set_project_access(self, project_path, access):
@@ -708,6 +820,7 @@ class MerginClient:
         project_path = mp.metadata["name"]
         local_version = mp.metadata["version"]
         server_info = self.project_info(project_path, since=local_version)
+
         pull_changes = mp.get_pull_changes(server_info["files"])
 
         push_changes = mp.get_push_changes()
