@@ -10,7 +10,7 @@ import pytz
 import sqlite3
 
 from .. import InvalidProject
-from ..client import MerginClient, ClientError, MerginProject, LoginError, decode_token_data, TokenError
+from ..client import MerginClient, ClientError, MerginProject, LoginError, decode_token_data, TokenError, ServerType
 from ..client_push import push_project_async, push_project_cancel
 from ..utils import (
     generate_checksum,
@@ -35,12 +35,16 @@ CHANGED_SCHEMA_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "
 
 @pytest.fixture(scope="function")
 def mc():
-    return create_client(API_USER, USER_PWD)
+    client = create_client(API_USER, USER_PWD)
+    create_workspace_for_client(client)
+    return client
 
 
 @pytest.fixture(scope="function")
 def mc2():
-    return create_client(API_USER2, USER_PWD2)
+    client = create_client(API_USER2, USER_PWD2)
+    create_workspace_for_client(client)
+    return client
 
 
 def create_client(user, pwd):
@@ -48,6 +52,13 @@ def create_client(user, pwd):
     return MerginClient(SERVER_URL, login=user, password=pwd)
 
 
+def create_workspace_for_client(mc):
+    try:
+        mc.create_workspace(mc.username())
+    except ClientError:
+        return
+
+
 def cleanup(mc, project, dirs):
     # cleanup leftovers from previous test if needed such as remote project and local directories
     try:
@@ -118,12 +129,13 @@ def test_create_remote_project_from_local(mc):
 
     # create remote project
     mc.create_project_and_push(test_project, directory=project_dir)
-
+    source_mp = MerginProject(project_dir)
     # check basic metadata about created project
     project_info = mc.project_info(project)
     assert project_info["version"] == "v1"
     assert project_info["name"] == test_project
     assert project_info["namespace"] == API_USER
+    assert project_info["id"] == source_mp.metadata["project_id"]
 
     versions = mc.project_versions(project)
     assert len(versions) == 1
@@ -191,6 +203,7 @@ def test_push_pull_changes(mc):
     f_remote_checksum = next((f["checksum"] for f in project_info["files"] if f["path"] == f_updated), None)
     assert generate_checksum(os.path.join(project_dir, f_updated)) == f_remote_checksum
     mp = MerginProject(project_dir)
+    assert project_info["id"] == mp.metadata["project_id"]
     assert len(project_info["files"]) == len(mp.inspect_files())
     project_versions = mc.project_versions(project)
     assert len(project_versions) == 2
@@ -292,7 +305,6 @@ def test_ignore_files(mc):
 
 
 def test_sync_diff(mc):
-
     test_project = f"test_sync_diff"
     project = API_USER + "/" + test_project
     project_dir = os.path.join(TMP_DIR, test_project)  # primary project dir for updates
@@ -330,6 +342,7 @@ def test_sync_diff(mc):
     # check project after push
     project_info = mc.project_info(project)
     assert project_info["version"] == "v3"
+    assert project_info["id"] == mp.metadata["project_id"]
     f_remote = next((f for f in project_info["files"] if f["path"] == f_updated), None)
     assert next((f for f in project_info["files"] if f["path"] == "renamed.gpkg"), None)
     assert not next((f for f in project_info["files"] if f["path"] == f_removed), None)
@@ -583,9 +596,17 @@ def test_available_storage_validation(mc):
     # download project
     mc.download_project(test_project_fullname, project_dir)
 
-    # get user_info about storage capacity
-    user_info = mc.user_info()
-    storage_remaining = user_info["storage"] - user_info["disk_usage"]
+    # get info about storage capacity
+    storage_remaining = 0
+
+    if mc.server_type() == ServerType.OLD:
+        user_info = mc.user_info()
+        storage_remaining = user_info["storage"] - user_info["disk_usage"]
+    else:
+        for workspace in mc.workspaces_list():
+            if workspace["name"] == API_USER:
+                storage_remaining = workspace["storage"] - workspace["disk_usage"]
+                break
 
     # generate dummy data (remaining storage + extra 1024b)
     dummy_data_path = project_dir + "/data"
@@ -597,8 +618,8 @@ def test_available_storage_validation(mc):
     try:
         mc.push_project(project_dir)
     except ClientError as e:
-        # Expecting "Storage limit has been reached" error msg.
-        assert str(e).startswith("Storage limit has been reached")
+        # Expecting "You have reached a data limit" 400 server error msg.
+        assert str(e) == "You have reached a data limit"
         got_right_err = True
     assert got_right_err
 
@@ -607,6 +628,9 @@ def test_available_storage_validation(mc):
     assert project_info["version"] == "v0"
     assert project_info["disk_usage"] == 0
 
+    # remove dummy big file from a disk
+    remove_folders([project_dir])
+
 
 def test_available_storage_validation2(mc, mc2):
     """
@@ -640,9 +664,14 @@ def test_available_storage_validation2(mc, mc2):
     # download project
     mc.download_project(test_project_fullname, project_dir)
 
-    # get user_info about storage capacity
-    user_info = mc.user_info()
-    storage_remaining = user_info["storage"] - user_info["disk_usage"]
+    # get info about storage capacity
+    storage_remaining = 0
+    if mc.server_type() == ServerType.OLD:
+        user_info = mc.user_info()
+        storage_remaining = user_info["storage"] - user_info["disk_usage"]
+    else:
+        # This test does not make sense in newer servers as quotas are tied to workspaces, not users
+        return
 
     # generate dummy data (remaining storage + extra 1024b)
     dummy_data_path = project_dir + "/data"
@@ -863,6 +892,7 @@ def test_get_versions_with_file_changes(mc):
 
     project_info = mc.project_info(project)
     assert project_info["version"] == "v4"
+    assert project_info["id"] == mp.metadata["project_id"]
     file_history = mc.project_file_history_info(project, f_updated)
 
     with pytest.raises(ClientError) as e:
@@ -897,6 +927,7 @@ def test_download_file(mc):
 
     project_info = mc.project_info(project)
     assert project_info["version"] == "v5"
+    assert project_info["id"] == mp.metadata["project_id"]
 
     # Versioned file should have the following content at versions 2-4
     expected_content = ("inserted_1_A.gpkg", "inserted_1_A_mod.gpkg", "inserted_1_B.gpkg")
@@ -926,6 +957,7 @@ def test_download_diffs(mc):
 
     project_info = mc.project_info(project)
     assert project_info["version"] == "v4"
+    assert project_info["id"] == mp.metadata["project_id"]
 
     # Download diffs of updated file between versions 1 and 2
     mc.get_file_diff(project_dir, f_updated, diff_file, "v1", "v2")
@@ -1743,13 +1775,7 @@ def test_project_versions_list(mc, mc2):
     # now user shold have write access
     assert mc.has_writing_permissions(test_project_fullname)
 
-    # test organization permissions
-    test_project_fullname = "testorg" + "/" + "test_org_permissions"
-
     # owner should have write access
-    assert mc.has_writing_permissions(test_project_fullname)
-
-    # writer should have write access
     assert mc2.has_writing_permissions(test_project_fullname)
 
 
